package wasm_of_ocaml-compiler

  1. Overview
  2. Docs

Module Wasm_of_ocaml_compiler.Wat_preprocessSource

Sourcetype value =
  1. | Bool of bool
  2. | String of string
  3. | Version of int * int * int
Sourceval value_equal : value -> value -> bool
Sourceval f : variables:(string * value) list -> filename:string -> contents:string -> string
Sourcetype source =
  1. | Binary
    (*

    Binary file (skipped by the preprocessor)

    *)
  2. | File
    (*

    Not read yet

    *)
  3. | Contents of string
    (*

    File contents to preprocess

    *)
Sourcetype input = {
  1. module_name : string;
    (*

    Name under which the module should be imported (used to fill the Binary.link_input record)

    *)
  2. file : string;
    (*

    File originally containing the module

    *)
  3. source : source;
    (*

    Information about the file, including possibly the already read file contents

    *)
}
Sourceval with_preprocessed_files : variables:(string * value) list -> inputs:input list -> (Binaryen.link_input list -> 'a) -> 'a